@@ -189,6 +189,13 @@ esa <- function (object, sessnum = 1, beta = NULL, real = NULL, noccasions = NUL
                 ## covariates(mask)$D <- D
                 stop ("userdist function requiring D not implemented for esa")
 
+            ## pass miscellaneous unmodelled parameter(s) 2015-02-21
+            nmiscparm <- length(object$details$miscparm)
+            if (nmiscparm > 0) {
+                miscindx <- max(unlist(object$parindx)) + (1:nmiscparm)
+                attr(mask, 'miscparm') <- coef(object)[miscindx, 1]
+            }
+
             distmat <- valid.userdist (object$details$userdist,
                                        detector(trps),
                                        xy1 = trps,
