@@ -11,8 +11,8 @@ usagePlot <- function(object, add = FALSE, occasion = NULL, col =
         if (!add)
             plot(object, ...)
 
+        nocc <- ncol(usage(object))
         if (is.null(occasion)) {
-            nocc <- ncol(usage(object))
             dx  <- rep((cos((1:nocc) * 2 * pi / nocc) * rad), each=nrow(object))
             dy  <- rep((sin((1:nocc) * 2 * pi / nocc) * rad), each=nrow(object))
             xy <- cbind(rep(object$x, nocc) + dx, rep(object$y, nocc)-dy)
@@ -20,7 +20,10 @@ usagePlot <- function(object, add = FALSE, occasion = NULL, col =
         }
         else {
             xy <- object
-            radius <- sqrt(usage(object)[,occasion]) * scale
+            if (tolower(occasion[1]) == "all")
+                occasion <- 1:nocc
+            usge <- apply(usage(object)[,occasion, drop = FALSE], 1, sum)
+            radius <- sqrt(usge) * scale
         }
 
         # metres: use symbols with inches = FALSE
